'********************************************************
'*							*
'*		Propeller II ROM Monitor		*
'*							*
'*		Version 0.3				*
'*							*
'********************************************************
'
' Version 0.1 - 11/01/2012 - original
' Version 0.2 - 09/22/2013 - added semicolon comments
' Version 0.3 - 11/13/2013 - began using SERA
'
'
' Usage:	cognew($6E4, rx_pin << 9 + tx_pin)	'start monitor in new cog
'
'		rx_pin bit 8 can be set for rx inversion
'		tx_pin bit 8 can be set for tx inversion
'

CON

  branch1_	= 0
  branch2_	= branch1_	+ 33
  branch3_	= branch2_	+ 35
  hello_	= branch3_	+ 15
  error_	= hello_	+ 34
  hitspace_	= error_	+ 11
  spquote_	= hitspace_	+ 10
  quotecr_	= spquote_	+ 4
  sub0_		= quotecr_	+ 3		'must be => $80
  sub1_		= sub0_		+ 4
  sub2_		= sub1_		+ 12
  sub3_		= sub2_		+ 12
  sub4_		= sub3_		+ 4
  sub5_		= sub4_		+ 4
  sub6_		= sub5_		+ 4
  sub7_		= sub6_		+ 8
  help_		= sub7_		+ 6


DAT

'********
'* Data *
'********

branch1		byte	cmd_new,	0
		byte	cmd_new,	";"
		byte	cmd_byte,	"Y"
		byte	cmd_word,	"W"
		byte	cmd_long,	"N"
		byte	cmd_viewp,	"."
		byte	cmd_search,	"/"
		byte	cmd_enter,	":"
		byte	cmd_map,	"M"
		byte	cmd_setp,	"H"
		byte	cmd_clrp,	"L"
		byte	cmd_notp,	"T"
		byte	cmd_offp,	"Z"
		byte	cmd_getp,	"R"
		byte	cmd_quit,	"Q"
		byte	cmd_help,	"?"
		byte	0			'33 bytes

branch2		byte	cmd_view2,	0
		byte	cmd_view2,	" "
		byte	cmd_range,	"."
		byte	cmd_search2,	"/"
		byte	cmd_enter2,	":"
		byte	cmd_watch,	"@"
		byte	cmd_clkset,	"*"
		byte	cmd_coginit,	"+"
		byte	cmd_cogstop,	"-"
		byte	cmd_setp,	"H"
		byte	cmd_clrp,	"L"
		byte	cmd_notp,	"T"
		byte	cmd_offp,	"Z"
		byte	cmd_getp,	"R"
		byte	cmd_watchp,	"#"
		byte	cmd_cfgp,	"|"
		byte	cmd_setdacs,	"\"
		byte	0			'35 bytes

branch3		byte	cmd_view3,	0
		byte	cmd_view3,	" "
		byte	cmd_search3,	"/"
		byte	cmd_enter3,	":"
		byte	cmd_move,	">"
		byte	cmd_move,	"<"
		byte	cmd_checksum,	"^"
		byte	0			'15 bytes

hello		byte	13,13,"=== Propeller 2.0 Monitor ===",13,13
		byte	0			'34 bytes

error		byte	"? - Help"
		byte	13,7,0			'11 bytes

hitspace	byte	"Hit SPACE",0		'10 bytes

spquote		byte	"  '",0			'4 bytes

quotecr		byte	"'",13,0		'3 bytes

sub0		byte	" -",13,$80		'4 bytes
sub1		byte	"{adr{.adr}}",$80	'12 bytes
sub2		byte	"{dat{ dat}}",$80	'12 bytes
sub3		byte	"adr",$80		'4 bytes
sub4		byte	"pin",$80		'4 bytes
sub5		byte	"dat",$80		'4 bytes
sub6		byte	"`Watch",13,$80		'8 bytes
sub7		byte	"`Set ",$80		'6 bytes

help		byte	13,		"~HUB",sub0_
		byte	sub1_,				"`View",13
		byte	sub1_,"/",sub2_,		"`Search",13
		byte	sub1_,":",sub2_,		"`Enter",13
		byte	sub3_,".",sub3_,"[</>]",sub3_,	"`Move",13
		byte	sub3_,".",sub3_,"^",		"`Checksum",13
		byte	sub3_,"@",			sub6_
		byte	"[Y/W/N]",			"`Byte/word/long",13
		byte			"~COGS",sub0_
		byte	"cog+",sub3_,"{+",sub3_,"}",	"`Start",13
		byte	"cog-",				"`Stop",13
		byte	"M",				"`Map",13
		byte			"~PINS",sub0_
		byte	"{",sub4_,"}[H/L/T/Z/R]",	"`High/low/toggle/off/read",13
		byte	sub4_,"#",			sub6_
		byte	sub4_,"|",sub5_,		"`Configure",13
		byte	sub5_,"\",			sub7_,"DACs",13
		byte			"~MISC",sub0_
		byte	sub5_,"*",			sub7_,"clock",13
		byte	"'",				"`Repeat",13
		byte	"Q",				"`Quit",13	'(0 long follows)

longs		long


'*********
'* Entry *
'*********

		org

entry		long	0			'start of data string = 0/nop

		reps	#$1F2-reserves,#1	'clear reserves
		setinda	#reserves
		mov	inda++,#0

		getptra	ctra			'get rx pin into ctra configuration
		testb	ctra,#16	wc	'handle !rx
		swbytes	ctra,#%%3313		'keep byte 1, clear others
		sets	ctra,#%1_100_01001	'ready state-time mode
		setbnc	ctra,#5			'handle !rx

		getptra	sera			'get rx/tx pins
		setb	sera,#8			'ready tx 8T/8N
		setb	sera,#17		'ready rx 8T/8N

		setp	sera			'tx high

		jmptask	#%0010,#baud_task	'enable baud detector task
		settask	#%%1010

		jz	base,#$			'wait for <space> to set period

		clrp	sera			'release tx

		jmptask	#%0100,#rx_task		'enable serial receiver task
		settask	#%%1210

		mov	wsize,#1		'init word size to byte
		call	#set_size

		wraux	#0,#0			'init input line to <enter>

		getptrb	base			'get base address of byte data
		sub	base,#longs<<2

		setptra	#hello_			'print hello message

dmax						'end of data string


'*************
'* Main Task *
'*************

message		call	#tx_string		'print hello/error message

cmd_new		call	#rx_line		'get input line

		call	#parse			'parse first term
	if_z	jz	x,#cmd_viewl		'if no hex and eol, view data
		jmp	#cmd_go			'else, process command


cmd_next_crlf	call	#tx_crlf		'print cr/lf
cmd_next	addspa	#1			'skip chr

cmd_loop	call	#parse			'parse next term
cmd_go	if_nz	jmp	#cmd_hex		'if hex, branch
		setd	pinx,#z			'pin update redirected to z
		setptra	#branch1_		'not hex, vector by chr
		call	#vector			'if returns, no match

cmd_error	setptra	#error_			'print error message
		jmp	#message


cmd_hex		mov	v1,value		'hex, save v1
		setd	pinx,#pin		'pin update okay
		setptra	#branch2_		'vector by chr
		call	#vector			'if returns, no match
		jmp	#cmd_view2		'view data


cmd_range	call	#parse_next		'hex., get hex
	if_z	jmp	#cmd_viewp2		'if no hex, view data

		mov	v2,value		'hex.hex, save v2
		setptra	#branch3_		'vector by chr
		call	#vector			'if returns, no match
		jmp	#cmd_view3		'view data
'
'
' Byte/word/long data
'
cmd_byte	mov	wsize,#1	wz	'set byte mode, z=0
cmd_word if_z	mov	wsize,#2	wz	'set word mode, z=0
cmd_long if_z	mov	wsize,#4		'set long mode

		call	#set_size

		jmp	#cmd_next		'next command
'
'
' View data
'
cmd_viewl	mov	v2,#$F			'<enter> (eol), show line of data
		call	#tx_range1
		jmp	#cmd_new

cmd_viewp	mov	v2,#$FF			'. (more), show page of data
		call	#tx_range1
		jmp	#cmd_next

cmd_viewp2	mov	v2,#$FF			'addr. (more), show page of data
		call	#tx_range2
		jmp	#cmd_loop

cmd_view2	mov	v2,v1			'addr, show unit of data
cmd_view3	call	#tx_range		'addr.addr, show range of data
		jmp	#cmd_loop
'
'
' Search
'
cmd_search	mov	v1,view			'/, search from view to end
cmd_search2	mov	v2,amask		'addr/, search from address to end
cmd_search3	call	#check_range		'addr.addr/, search range

		call	#parse_data		'parse data string

:start		setptra	v1			'start search, point to search address
		setinda	#0			'point to search data
		mov	x,#0			'reset word match counter

:word		call	#rdxxxx			'get memory word
		cmp	value,inda++	wz	'compare against search data word
	if_z	jmp	#:match			'if word match, check if string match

		add	v1,wsize		'word mismatch, advance search address
		cmp	v1,v2		wz,wc	'at end of range?
	if_be	jmp	#:start			'if not, start next search

		mov	view,v1			'else, update view
		and	view,amask
		jmp	#cmd_loop		'next command

:match		incmod	x,dsize		wc	'word match, increment match counter
	if_nc	jmp	#:word			'if more words to match, compare next word

		mov	v2,dsize		'got string match
		shl	v2,shift		'v1 = start of found data
		add	v2,v1			'v2 = end of found data
		jmp	#cmd_view3		'show found data
'
'
' Enter data
'
cmd_enter3	call	#check_range		'addr.addr:, check range, z=words to fill
		mov	y,#1			'set fill flag

cmd_enter2	mov	enter,v1		'addr:, set enter address
		and	enter,amask		'trim enter address (in case cmd_enter2)

cmd_enter	call	#parse_data		':, parse data

		jnz	y,#:go			'if not fill, set z to data size
		mov	z,dsize
		add	z,#1

:go		setptrb	enter			'get pointer

:loop		mov	value,inda++		'get word from string data
		call	#wrxxxx			'store value in memory
		djnz	z,#:loop		'loop until enter done

		getptrb	enter			'update pointer

		jmp	#cmd_loop		'next command
'
'
' Move data
'
cmd_move	mov	y,x			'save ">"/"<"

		call	#check_range		'check 1st address range, get number of words

		call	#parse_hex		'get 2nd address

		max	value,amask		'v1=1st, value=2nd, z=words, y=">"/"<"
		and	value,amask

		cmp	y,#"<"		wz	'if "<", swap v1 and value
	if_z	mov	x,v1
	if_z	mov	v1,value
	if_z	mov	value,x			'v1=from, value=to, z=words

		cmp	v1,value	wc	'if from < to, downward move
	if_c	mov	x,z
	if_c	shl	x,shift
	if_c	add	v1,x
	if_c	add	value,x
	if_c	xor	rdxxxx,#%001_111110	'modify 'rdxxxx value,--ptra'
	if_c	xor	wrxxxx,#%001_111110	'modify 'wrxxxx value,--ptrb'

		setptra	v1			'set pointers
		setptrb	value

:loop		call	#rdxxxx			'move data
		call	#wrxxxx
		djnz	z,#:loop

	if_c	xor	rdxxxx,#%001_111110	'restore 'rdxxxx value,ptra++'
	if_c	xor	wrxxxx,#%001_111110	'restore 'wrxxxx value,ptrb++'

		jmp	#cmd_loop		'next command
'
'
' Checksum
'
cmd_checksum	call	#check_range		'check range

		setptra	v1			'sum bytes/words/longs
:loop		call	#rdxxxx
		add	y,value
		djnz	z,#:loop

		mov	value,y			'print sum
		mov	hsize,#8
		call	#tx_hex

		jmp	#cmd_next_crlf		'next command
'
'
' Watch
'
cmd_watchp	setx	rdxxxj,#%01111		'set pin mode
		mov	hsize,#1	wz	'set single hex digit, z=0

cmd_watch if_z	setx	rdxxxj,#%00000		'set mem mode
	  if_z	incpat	hsize,wsize		'set hex size by word size

		call	#rdxxxp			'get initial value

:loop		mov	z,value			'preserve value

		call	#tx_hex			'print value
		serouta	#" "			'print space

:wait		call	#rx_check		'if key hit, exit
	if_nz	jmp	#cmd_next_crlf

		call	#rdxxxp			'get current value

		cmp	value,z		wz	'if same, check again
	if_z	jmp	#:wait

		jmp	#:loop			'new value, loop
'
'
' Clkset
'
cmd_clkset	setptra	#hitspace_		'print hit-space message
		call	#tx_string

		clkset	v1			'set clk

:wait		call	#rx			'wait for space
		cmp	x,#" "		wz
	if_nz	jmp	#:wait

		jmp	#cmd_next_crlf		'next command
'
'
' Coginit
'
cmd_coginit	setnib	:cog,v1,#6		'set cog

		call	#parse_hex		'get program address
		mov	y,value

		mov	value,#0		'set default pointer address
		cmp	x,#"+"		wz	'if '+', get pointer address
	if_z	call	#parse_hex

:cog		coginit	y,value,#0		'do 'coginit program,pointer'

		jmp	#cmd_loop		'next command
'
'
' Cogstop
' Quit
'
cmd_quit	cogid	v1			'quit

cmd_cogstop	cogstop	v1			'stop cog

		jmp	#cmd_next		'next command
'
'
' Map
'
cmd_map		mov	y,#7			'ready for 7..0

cmd_map_loop	serouta	#" "			'print space
		cogid	x			'get cog id
		cogid	y		wc	'get cog y status
cmd_map_c	cmp	x,y		wz
	if_nc	mov	x,#"0"			'get 0/1/M chr
	if_c	mov	x,#"1"
	if_z	mov	x,#"M"
		serouta	x			'print chr
		sub	y,#1		wc
	if_nc	jmp	#cmd_map_loop		'loop until done

		jmp	#cmd_next_crlf		'next command
'
'
' Pin writes clrp/setp/notp/offp
' Pin read
'
cmd_setp	mov	x,#$9B		wz	'setp, z=0
cmd_clrp if_z	mov	x,#$9A		wz	'clrp, z=0
cmd_notp if_z	mov	x,#$99		wz	'notp, z=0
cmd_offp if_z	mov	x,#$98		wz	'offp, z=0
cmd_getp if_z	mov	x,#$30			'getp, z=1

		sets	pinop,x

pinx		mov	pin,v1			'if hex, get pin (d = pin/z)

pinop		getp	pin		wc	'becomes setp/clrp/notp/offp/getp

	if_z	jmp	#cmd_map_c		'if getp, show pin value

		jmp	#cmd_next		'next command
'
'
' Pin configuration
'
cmd_cfgp	call	#parse_hex		'get configuration

		decod5	x,v1		wz,wc	'get pin mask
		setbz	:i,#25			'set pin port
		setbc	:i,#24
:i if_nz_or_nc	cfgpins	x,value,#0		'configure pin if $00..$5F

		jmp	#cmd_loop		'next command
'
'
' Setdacs
'
cmd_setdacs	setdacs	v1			'set all four dacs with 8-bit values

		jmp	#cmd_next		'next command
'
'
' Help
'
cmd_help	setptra	#help_			'print help message
		call	#tx_string

		jmp	#cmd_next_crlf		'next command


'*************************
'* Main Task Subroutines *
'*************************
'
'
' Vector branch
'
vector		addptra	base			'add data base pointer

vector_loop	rdbyte	z,ptra++		'get jump address
vector_ret	jz	z,#0			'if 0, no match found, return

		rdbyte	y,ptra++		'get target
		xor	y,x		wz	'compare to x
	if_nz	jmp	#vector_loop		'if no match, loop

		jmp	z			'match found, jump, y=0, z=1
'
'
' Check address range (v1..v2)
'
check_range	max	v1,amask		'trim v1
		and	v1,amask

		max	v2,amask		'trim v2
		and	v2,amask

		cmp	v2,v1		wc	'make sure v2 => v1
	if_c	jmp	#cmd_error

		mov	z,v2			'get number of words
		sub	z,v1
		shr	z,shift
		add	z,#1

check_range_ret	ret
'
'
' Set rdxxxx/wrxxxx and others by word size
'
set_size	test	wsize,#%010	wc	'set rdxxxx/wrxxxx by word size
		setbc	rdxxxx,#26
		setbc	wrxxxx,#24

		test	wsize,#%100	wc
		setbc	rdxxxx,#27
		setbc	wrxxxx,#25

		mov	shift,wsize		'set shift by word size
		shr	shift,#1

		neg	amask,wsize		'set amask by word size
		and	amask,h0001FFFF

		and	view,amask		'trim view
		and	enter,amask		'trim enter

set_size_ret	ret

h0001FFFF	long	$0001FFFF		'memory limit


rdxxxp		getp	v1		wc	'read pin
	if_nc	mov	value,#0
	if_c	mov	value,#1
rdxxxj		jmp	rdxxxx_ret		'condition = t/f

		setptra	v1			'read mem

rdxxxx		rdbyte	value,ptra++		'rdbyte/rdword/rdlong
rdxxxp_ret
rdxxxx_ret	ret


wrxxxx		wrbyte	value,ptrb++		'wrbyte/wrword/wrlong

wrxxxx_ret	ret
'
'
' Input line
'
rx_line		setspa	#0			'point to start of line

		serouta	#">"			'show prompt

		call	#rx			'get first chr
		cmp	x,#"'"		wz	'check for repeat
	if_nz	jmp	#:first			'if not repeat, first chr

:show		rdaux	x,spa++		wz	'repeat, show line
	if_nz	serouta	x
	if_nz	jmp	#:show
		jmp	#:done


:loop		call	#rx			'get next chr

:first		cmp	x,#13		wz	'cr?
	if_z	jmp	#:cr

		cmp	x,#8		wz	'backspace?
	if_nz	cmp	x,#127		wz
	if_z	jmp	#:bs

		cmp	x,#" "		wc	'visible chr?
	if_nc	cmpr	x,#"~"		wc
	if_c	jmp	#:loop

		wraux	x,spa++			'visible chr, append to line
		chkspa			wc	'overflow?
	if_c	subspa	#1			'if overflow, back up
	if_nc	serouta	x			'if not overflow, print chr
		jmp	#:loop

:bs		chkspa			wz	'backspace, line empty? if not,
	if_nz	serouta	x			'..print backspace
	if_nz	serouta	#" "			'..print space
	if_nz	serouta	x			'..print backspace
	if_nz	subspa	#1			'..back up
		jmp	#:loop

:cr		wraux	#0,spa			'cr, end line with 0

:done		setspa	#0			'point to start of line

tx_crlf		serouta	#$0D			'print cr/lf
		serouta	#$0A
tx_crlf_ret
rx_line_ret	ret
'
'
' Parse hex/text data
'
parse_data	mov	w,#0			'reset data count
		setinda	#0			'point to string data

:hex		call	#parse_next		'hex loop, check hex
	if_nz	call	#enter_data		'if hex, enter value
		cmp	x,#" "		wz	'check for space (more hex)
	if_z	jmp	#:hex			'if more hex, loop

		cmp	x,#"'"		wz	'not hex, "'"?
	if_nz	jmp	#:done			'if not "'", done

:text		rdaux	x,++spa			'text loop, get and point to next chr
		cmp	x,#"'"		wz	'check for "'"
	if_z	jmp	#:hex			'if "'", back to hex
		jz	x,#:done		'if eol, done
		mov	value,x			'text chr
		call	#enter_data		'enter chr
		jmp	#:text			'loop

:done		sub	w,#1		wc	'get data count
	if_nc	mov	dsize,w			'if 0, reuse old data

		setd	:fix,dsize		'form circular buffer
:fix		fixinda	#0,#0			'(no instruction-modification problem with 1:4 threading)
	
parse_data_ret	ret


enter_data	incmod	w,#dmax		wc	'check if data limit exceeded
	if_c	jmp	#cmd_error		'if data limit exceeded, error
		mov	inda++,value		'store value in data

enter_data_ret	ret
'
'
' Parse hex
'
parse_hex	call	#parse_next		'try to parse hex
	if_z	jmp	#cmd_error		'if no hex, error

parse_hex_ret	ret
'
'
' Parse line (@spa), z=0 if hex (value)
'
parse_next	addspa	#1			'advance to next chr

parse		mov	value,#0	wz	'z=1

		call	#skip_spaces	wz	'skip any spaces (preserve z)

:loop		rdaux	x,spa++			'get chr
		call	#check_hex		'check hex
	if_c	rolnib	value,x			'if hex, append nibble and loop
	if_c	jmp	#:loop		wz	'z=0

		subspa	#1			'repoint to non-hex chr

		call	#skip_spaces	wz	'skip any post-hex spaces (preserve z)

		call	#check_hex		'check hex
	if_c	rdaux	x,--spa			'if hex, back up to space chr

		cmpr	x,#"a"-1	wc	'make non-hex chr uppercase
	if_c	cmp	x,#"z"+1	wc
	if_c	sub	x,#"a"-"A"
parse_next_ret
parse_ret	ret
'
'
' Skip spaces (@spa)
'
skip_spaces	rdaux	x,spa++			'skip space chr(s)
		cmp	x,#" "		wz
	if_z	jmp	#skip_spaces

		subspa	#1			'back up to non-space chr

skip_spaces_ret	ret			wz	'restore z
'
'
' Check hex (x), c=1 if hex (x)
'
check_hex	cmpr	x,#"0"-1	wc	'"0".."9" -> $0..$9
	if_c	cmp	x,#"9"+1	wc
	if_c	add	x,#"A"-"9"-1

	if_nc	cmpr	x,#"A"-1	wc	'"A".."F" -> $A..$F
	if_c	cmp	x,#"F"+1	wc
	if_c	add	x,#"a"-"A"

	if_nc	cmpr	x,#"a"-1	wc	'"a".."f" -> $A..$F
	if_c	cmp	x,#"f"+1	wc
	if_c	sub	x,#"a"-10

check_hex_ret	ret
'
'
' Print range (v1..v2)
'
tx_range1	mov	v1,view			'view..view + v2
tx_range2	and	v2,amask		'v1..v1 + v2
		add	v2,v1

tx_range	call	#check_range		'check range

		mov	view,v1			'set address

:line		mov	value,view		'print 5-digit address
		mov	hsize,#5
		call	#tx_hex

		serouta	#"-"			'print "- "
		serouta	#" "

		mov	x,wsize			'get number of words on line
		rev	x,#32-5
		mov	v1,z
		max	v1,x

		mov	v2,v1			'get number of ascii bytes on line
		shl	v2,shift

		sub	z,v1			'update number of words left

		setptra	view			'print hex words
:hex		call	#rdxxxx
		incpat	hsize,wsize
		call	#tx_hex
		serouta	#" "
		djnz	v1,#:hex

		setptra	#spquote_		'print "  '"
		call	#tx_string

		setptra	view			'print ascii bytes
:ascii		rdbyte	x,ptra++
		cmp	x,#" "		wc	'visible chr?
	if_nc	cmpr	x,#"~"		wc
	if_c	mov	x,#"."			'substitute "." for non-visible chrs
		serouta	x
		djnz	v2,#:ascii

		getptra	view			'update address

		setptra	#quotecr_		'print "'" + cr
		call	#tx_string

		call	#rx_check		'check key hit
	if_z	jnz	z,#:line		'if no key hit and more words left, print another line
tx_range1_ret
tx_range2_ret
tx_range_ret	ret
'
'
' Print string (@ptra)
'
tx_string	addptra	base			'add data base pointer

tx_str		rdbyte	x,ptra++		'get chr

tx_string_ret	jz	x,#0			'if 0, done

		test	x,#$80		wz	'substring?
	if_nz	notb	tx_str,#8		'toggle ptra/ptrb
	if_nz	setptrb	x			'ptrb points to substring
	if_nz	addptrb	base
	if_nz	jmp	#tx_str			'start substring or resume string

		cmp	x,#"`"		wz	'long tab?
	if_z	subr	y,#32-16
	if_nz	cmp	x,#"~"		wz	'short tab?
	if_z	add	y,#16
:tab	if_z	serouta	#" "
	if_z	djnz	y,#:tab
	if_z	serouta	#"-"
	if_z	serouta	#" "
	if_z	jmp	#tx_str

		cmp	x,#13		wz	'cr?
	if_z	call	#tx_crlf
	if_z	mov	y,#0

	if_nz	serouta	x			'other?
	if_nz	add	y,#1
		jmp	#tx_str
'
'
' Print hex (value)
'
tx_hex		mov	y,hsize			'pre-rotate to get 1st nibble in top
		shl	y,#2
		ror	value,y

		mov	y,hsize			'print nibbles
:loop		rol	value,#4
		mov	x,value
		call	#tx_nib
		djnz	y,#:loop

tx_hex_ret	ret
'
'
' Print nibble (x)
'
tx_nib		and	x,#$F			'isolate nibble

		cmp	x,#$A		wc	'alpha or numeric?
	if_c	add	x,#"0"			'numeric
	if_nc	add	x,#"A"-$A		'alpha
		serouta	x

tx_nib_ret	ret
'
'
' Receive chr (x)
'
rx		call	#rx_check		'wait for rx chr
	if_z	jmp	#rx

rx_ret		ret
'
'
' Check receiver, z=0 if chr (x)
'
rx_check	cmp	rx_head,rx_tail	wz

	if_nz	rdauxr	x,rx_tail
	if_nz	incmod	rx_tail,#$7F

rx_check_ret	ret


'************************
'* Serial Receiver Task *
'************************

rx_task		serina	rx_data			'wait for character

		wrauxr	rx_data,rx_head		'store byte at head
		incmod	rx_head,#$7F		'inc head

		jmp	#rx_task		'wait for next byte


'**********************
'* Baud Detector Task *
'**********************

baud_task	notb	ctra,#8		wc	'if 1,0 sample set, c=0 (bit8 is not used by ctra)
		notb	ctra,#5			'toggle state to measure ($20 -> 10000001001 -> 1, 6x 0, 1x 1, 2x 0, 1)
		setctra	ctra			'ctra measures rx pin states

	if_nc	mov	limh,buf0		'if 1,0 sample set,
	if_nc	shr	limh,#4			'..make window from 1st 0 (6x if $20)
	if_nc	neg	liml,limh
	if_nc	add	limh,buf0
	if_nc	add	liml,buf0

	if_nc	mov	comp,buf1		'if 1,0 sample set,
	if_nc	mul	comp,#6			'..normalize 2nd 1 (1x if $20) to 6x
	if_nc	cmpr	comp,limh	wc	'..check if within window
	if_nc	cmp	comp,liml	wc

	if_nc	mov	comp,buf2		'if 1,0 sample set,
	if_nc	mul	comp,#3			'..normalize 2nd 0 (2x if $20) to 6x
	if_nc	cmpr	comp,limh	wc	'..check if within window
	if_nc	cmp	comp,liml	wc

	if_nc	add	buf0,buf2		'if $20,
	if_nc	shr	buf0,#3			'..compute period from 6x 0 and 2x 0
	if_nc	setsera	sera,buf0		'..(re)initialize serial
	if_nc	setb	base,#31		'..flag initial $20

		mov	buf0,buf1		'scroll sample buffer
		mov	buf1,buf2

:wait		getcosa	buf2			'wait for next sample
		jnz	buf2,#baud_task
		jmp	#:wait


'*************
'* Variables *
'*************

reserves

base		res	1			'main task
w		res	1
x		res	1
y		res	1
z		res	1
v1		res	1
v2		res	1
value		res	1
view		res	1
enter		res	1
pin		res	1
dsize		res	1
hsize		res	1
wsize		res	1
shift		res	1
amask		res	1

rx_data		res	1			'serial receiver task
rx_head		res	1
rx_tail		res	1

ctra		res	1			'baud detector task
sera		res	1
buf0		res	1
buf1		res	1
buf2		res	1
limh		res	1
liml		res	1
comp		res	1
