'**************************************
'*  NTSC 256 x 192 - luma/color bars  *
'**************************************

CON

  s		= 43		'scales DAC output (s = 0..128)
  r		= s * 78 / 128	'adjusts for modulator expansion

  lntsc         = 3640          'NTSC colorburst cycles per line * 16
  sntsc         = 624           'NTSC colorburst cycles per sync * 16


  v_bo		= (lntsc-sntsc-256*9)/2	'left/right border

  v_bs		= 100			'before sync		horizontal sync
  v_sn		= 269			'sync
  v_bc		= 50			'before colorburst
  v_cb		= 144			'colorburst
  v_bv		= sntsc-144-50-269-100	'before visible

  v_hs		= 135			'high sync		high vertical sync
  v_hr		= lntsc/2-135-100	'high sync remainder

  v_ls		= 1564			'low sync		low vertical sync
  v_lr		= lntsc/2-1564-100	'low sync remainder

  v_bl		= lntsc-sntsc		'blank visible

  c_sh		= $000000_01		'sync high
  c_sl		= $000000_02		'sync low


DAT		org

		clkset	#$FF			'set 80MHz

		reps	#64,#2			'start 8-bit incrementing sequence at $BB
		setspa	#$BB
		pusha	luma
		add	luma,lumad

		setctra	#%00111			'set ctra for PLL*8
                setfrqa	frqa_			'set frqa

		setvid	#%1110000		'set vid for composite output
		setvidy	vidy_			'set yiq coefficients
		setvidi	vidi_
		setvidq	vidq_

		cfgdac0	#%11			'set bottom dac to video mode (YIQ)
'
'
' Field loop
'
field           mov	x,#34			'top blanks
		call	#blank

                mov     x,#192			'set visible lines
line	        call	#hsync			'do horizontal sync
		waitvid	#v_bo,#0		'black left border
		waitvid	mode,#0			'256*9 clocks per waitvid - show luma/colorbars
		waitvid	#v_bo,#0		'black right border
		djnz    x,#line           	'another line?

                mov     x,#28			'bottom blanks
		call	#blank

		mov	x,#6			'high vertical syncs
		call	#vsynch

		mov	x,#6			'low vertical syncs
		call	#vsyncl

		mov	x,#6			'high vertical syncs
		call	#vsynch

                jmp     #field                  'field loop



blank		call	#hsync			'blank lines
		waitvid	#v_bl,#0
		djnz	x,#blank
blank_ret	ret


hsync		waitvid	#v_bs,#c_sh		'horizontal sync
		waitvid	#v_sn,#c_sl
hsync_ret	retd
		waitvid	#v_bc,#c_sh
		waitvid	#v_cb,c_cb
		waitvid	#v_bv,#c_sh


vsynch		waitvid	#v_bs,#c_sh		'vertical sync high
		waitvid	#v_hs,#c_sl
		waitvid	#v_hr,#c_sh
		djnz	x,#vsynch
vsynch_ret	ret


vsyncl		waitvid	#v_bs,#c_sh		'vertical sync low
		waitvid	#v_ls,#c_sl
		waitvid	#v_lr,#c_sh
		djnz	x,#vsyncl
vsyncl_ret	ret
'
'
' Initialized data
'
luma		long	$03020100
lumad		long	$04040404

frqa_		long	$16E8_B9FD		'57.27272MHz (colorburst * 16) @80MHz clock

vidy_		long	(110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_		long	(100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_		long	              128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)

		'	color base	mode		clocks/pixel	clocks/waitvid
mode		long	$BB << 24 +	$A << 20 +	9 << 13 +	256*9

c_cb		long	$687800_01		'colorburst reference color

x		long	0
