'******************************
'*  NTSC 256 x 192 - texture  *
'******************************

CON

  s		= 46		'scales DAC output (s = 0..128)
  r		= s * 78 / 128	'adjusts for modulator expansion

  lntsc         = 3640          'NTSC colorburst cycles per line * 16
  sntsc         = 624           'NTSC colorburst cycles per sync * 16

  v_bo		= (lntsc-sntsc-256*9)/2	'left/right border

  v_bs		= 100			'before sync		horizontal sync
  v_sn		= 269			'sync
  v_bc		= 50			'before colorburst
  v_cb		= 144			'colorburst
  v_bv		= sntsc-144-50-269-100	'before visible

  v_hs		= 135			'high sync		high vertical sync
  v_hr		= lntsc/2-135-100	'high sync remainder

  v_ls		= 1564			'low sync		low vertical sync
  v_lr		= lntsc/2-1564-100	'low sync remainder

  v_bl		= lntsc-sntsc		'blank visible

  c_sh		= $000000_01		'sync high
  c_sl		= $000000_02		'sync low


DAT		org

		clkset	#$FF			'set 80MHz

		reps	#10,#1			'install lookup colors into stack ram at $00
		setinda	#color0
		pusha	inda++

		setpix	pix_			'set 16x32 4-bit texture w/horizontal mirroring
		setmix	#$0BA,#0

		setctra	#%00111			'set ctra for PLL*8
                setfrqa	frqa_			'set frqa

		setvid	#%1110000		'set vid for composite output
		setvidy	vidy_			'set yiq coefficients
		setvidi	vidi_
		setvidq	vidq_

		cfgdac0	#%11			'set bottom dac to video mode (YIQ)
'
'
' Field loop
'
field           mov	x,#34			'top blanks
		call	#blank

		setpixz	pixz_			'reset pix registers
		setpixu	pixu_
		setpixv	pixv_
		setpixa	pixa_
		setpixr	pixr_
		setpixg	pixg_
		setpixb	pixb_


                mov     x,#192			'set visible lines
line	        call	#hsync			'do horizontal sync
		waitvid	#v_bo,#0		'black left border


		waitvid	mode,#$80		'64*9 clocks per waitvid
		setspa	#$80
		call	#render			'do waitvid and render 1st 64 pixels

		waitvid	mode,#$C0		'64*9 clocks per waitvid
		setspa	#$C0
		call	#render			'do waitvid and render 2nd 64 pixels

		waitvid	mode,#$80		'64*9 clocks per waitvid
		setspa	#$80
		call	#render			'do waitvid and render 3rd 64 pixels

		waitvid	mode,#$C0		'64*9 clocks per waitvid
		setspa	#$C0
		call	#render			'do waitvid and render 4th 64 pixels


		waitvid	#v_bo,#0		'black right border
		djnz    x,#line           	'another line?


                mov     x,#28			'bottom blanks
		call	#blank

		mov	x,#6			'high vertical syncs
		call	#vsynch

		mov	x,#6			'low vertical syncs
		call	#vsyncl

		mov	x,#6			'high vertical syncs
		call	#vsynch

                jmp     #field                  'field loop



blank		call	#hsync			'blank lines
		waitvid	#v_bl,#0
		djnz	x,#blank
blank_ret	ret


hsync		waitvid	#v_bs,#c_sh		'horizontal sync
		waitvid	#v_sn,#c_sl
hsync_ret	retd
		waitvid	#v_bc,#c_sh
		waitvid	#v_cb,c_cb
		waitvid	#v_bv,#c_sh


vsynch		waitvid	#v_bs,#c_sh		'vertical sync high
		waitvid	#v_hs,#c_sl
		waitvid	#v_hr,#c_sh
		djnz	x,#vsynch
vsynch_ret	ret


vsyncl		waitvid	#v_bs,#c_sh		'vertical sync low
		waitvid	#v_ls,#c_sl
		waitvid	#v_lr,#c_sh
		djnz	x,#vsyncl
vsyncl_ret	ret
'
'
' Render texture pixels
'
' Because WAITVID's D and S are double-buffered, the stream mode data is not actually
' read until the prior WAITVID is done. This means that the streaming WAITVID command
' can be issued, and then the stack RAM data can be quickly filled in.
'
' If you alternately switch between two stack RAM areas, you can avoid read/write
' conflicts with VID.
'
render		repd	#64,#1			'render texture pixels
		setinda	#pixels			'ready to blend 64 pixels
		wait	#3			'ready pipeline
		wait	#3			'ready pipeline
		getpix	inda++			'repeat getpix

		reps	#64,#1			'push pixels
		setinda	#pixels
		pusha	inda++

		reps	#64,#3			'reset pixels to background pattern
		setinda	#pixels
		incmod	lines,#7	wc
	if_c	xor	color,color_tog
		mov	inda++,color

render_ret	ret
'
'
' Initialized data - video
'
frqa_		long	$16E8_B9FD		'57.27272MHz (colorburst * 16) @80MHz clock

vidy_		long	(110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_		long	(100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_		long	              128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)

		'	color base	mode		clocks/pixel	clocks/waitvid
mode		long	$00 << 24 +	$F << 20 +	9 << 13 +	64 * 9

c_cb		long	$687800_01		'colorburst reference color

x		long	0
'
'
' Initialized data - texture mapping
'
color0		long	$00_00_00_00	'transparent black	clear
color1		long	$FF_D0_00_00	'solid red		propeller
color2		long	$FF_80_80_00	'solid olive		propeller shaft
color3		long	$FF_00_00_80	'solid dark blue	hat rim
color4		long	$FF_00_A0_FF	'solid cyan		hat outside
color5		long	$FF_00_78_C0	'solid cyan shadow	hat inside
color6		long	$FF_FF_FF_00	'solid yellow		hat outside
color7		long	$FF_C0_C0_00	'solid yellow shadow	hat inside
color8		long	$FF_FF_80_00	'solid orange		hat outside
color9		long	$FF_C0_60_00	'solid orange shadow	hat inside

color_tog	long	$1F_1F_1F
color		long	$FF_FF_FF
lines		long	0

		'       16 x 32 map, 4-bit pixels, long colors, horizontal mirroring
pix_		long	%100_101_10_0_10 << 21 + $00 << 9 + texture

pixz_		long	$0000_0100	'try %0000_0000, $8000_0000, $C000_0000
pixu_		long	$0000_0200	'change these values to experiment
pixv_		long	$0000_0001
pixa_		long	$FFFF_FF00
pixr_		long	$FFFF_FFFF
pixg_		long	$FFFF_FFFF
pixb_		long	$FFFF_FFFF

texture		long	$00000000,$00000000	'16x32 4-bit texture, mirrored horizontally to make 32x32 texture
		long	$11110000,$00000011
		long	$11111100,$20111111
		long	$11111110,$11111111
		long	$11111110,$20001111
		long	$11111000,$20000001
		long	$00000000,$66000000
		long	$00000000,$66000000
		long	$00000000,$20000000
		long	$00000000,$44440000
		long	$00000000,$86644440
		long	$40000000,$88664444
		long	$44000000,$88666444
		long	$44400000,$88866644
		long	$44440000,$88866664
		long	$44444000,$88866664
		long	$44444000,$88866666
		long	$44444400,$88886666
		long	$64444400,$88886666
		long	$64444400,$88886666
		long	$64444440,$88886666
		long	$66444440,$88886666
		long	$66444440,$33333666
		long	$33344440,$33333333
		long	$33333440,$99993333
		long	$77533330,$99997777
		long	$77555330,$99997777
		long	$77333300,$99997777
		long	$33330000,$99993333
		long	$33000000,$33333333
		long	$00000000,$33333000
		long	$00000000,$00000000

pixels		long	0[64]
